# -*- coding: utf-8 -*-
from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import DatabasePlugin
from ..utils import parse_int


class McManiac(DatabasePlugin):
    requires = ['irc3.plugins.command',
                'nxy.plugins.database']

    # noinspection PyUnusedLocal
    @command(options_first=True)
    def mcmaniac(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Get, add or delete McManiaCs
    
        %%mcmaniac <cmd> <mcmaniac>
        %%mcmaniac [<index>]
        """
        cmd = args.get('<cmd>')
        item = args.get('<mcmaniac>')
        if cmd and item:
            if self.guard.has_permission(mask, 'admin'):
                if cmd == 'add':
                    self.cur.execute('insert into mcmaniacs (item) values (?)',
                                     [item])
                if cmd == 'del':
                    index, order, op = parse_int(item, select=False)
                    if not index:
                        return
                    self.cur.execute('''delete from mcmaniacs where id =
                    (select id from mcmaniacs a where ? = (select count(id)
                    from mcmaniacs b where a.id {op} b.id) order by id {order})
                    '''.format(op=op, order=order), [index])
                self.con.commit()
            else:
                self.bot.notice(mask.nick, 'Permission denied')
        else:
            index = args.get('<index>')
            if index:
                index = parse_int(index)
                if not index:
                    return
                index, order, _ = index
                order = 'id {order}'.format(order=order)
                extra = 'offset ?'
                binds = [index]
            else:
                order = 'random()'
                extra = ''
                binds = []
            self.cur.execute('''select item,
            (select count(id) from mcmaniacs b where a.id >= b.id) as idx,
            (select count(id) from mcmaniacs) as len
            from mcmaniacs a order by {order} limit 1 {extra}
            '''.format(order=order, extra=extra), binds)
            result = self.cur.fetchone()
            if result:
                return '[{idx}/{len}] {item}'.format(**result)
